package dev.fyloz.musicplayer.core.factory

import dev.fyloz.musicplayer.core.http.auth.AuthorizationData
import dev.fyloz.musicplayer.core.model.SearchResult
import dev.fyloz.musicplayer.core.model.ExternalTrack
import dev.fyloz.musicplayer.core.model.Track

class TrackFactoryProxy {
    private val factories = mutableMapOf<String, TrackFactory>()

    fun registerFactory(source: String, factory: TrackFactory) {
        factories[source] = factory;
    }

    suspend fun search(query: String, auth: AuthorizationData): SearchResult {
        val results = mutableMapOf<String, Collection<ExternalTrack>>()
        factories.forEach { (source, factory) ->
            results[source] = factory.search(query, auth)
        }
        return results.toMap()
    }

    suspend fun create(source: String, id: String, trackId: String, auth: AuthorizationData): Track =
        getFactory(source).create(id, trackId, auth)

    private fun getFactory(source: String) = factories[source]!!;
}
