package dev.fyloz.musicplayer.core.http

import io.ktor.client.*
import io.ktor.client.call.*
import io.ktor.client.request.*
import io.ktor.client.statement.*
import io.ktor.http.*
import org.koin.core.component.KoinComponent
import org.koin.core.component.inject

abstract class HttpProvider(private val baseUrl: String) : KoinComponent {
    protected val httpClient by inject<HttpClient>()
    protected open val acceptContentType = ContentType.Application.Json

    protected suspend inline fun <reified T> get(
        path: String,
        bearer: String? = null,
        block: HttpRequestBuilder.() -> Unit = {}
    ): T =
        httpClient.get(path.toUrl()) {
            accept(acceptContentType)

            headers {
                if (bearer != null) {
                    append(HttpHeaders.Authorization, bearer.toBearer())
                }
            }

            block()
        }.process()

    protected suspend inline fun <reified T> HttpResponse.process(): T {
        if (this.status.isSuccess()) {
            return this.body()
        }

        throw IllegalStateException("TODO: Unsuccessful HTTP request: ${this.body<String>()}")
    }

    protected fun String.toUrl(): String {
        if (baseUrl.endsWith('/') && this.startsWith('/')) {
            return baseUrl + this.substring(1, this.length)
        }

        if (!this.startsWith('/')) {
            return "$baseUrl/$this"
        }

        return baseUrl + this
    }

    protected fun String.toBearer() = "Bearer $this"
}