package dev.fyloz.musicplayer.core.http

import dev.fyloz.musicplayer.core.getAuthorizationData
import dev.fyloz.musicplayer.core.http.requests.ImportTrackRequest
import dev.fyloz.musicplayer.core.logic.TrackLogic
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import org.koin.ktor.ext.inject

fun Route.configureTrackRoutes() {
    val logic by inject<TrackLogic>()

    route("/tracks") {
        get("/") {
            call.respond(logic.getAll().toList())
        }

        get("/trackIds/") {
            call.respond(logic.getAllTrackIdsBySource())
        }

        get("/search") {
            val query = call.request.queryParameters["q"]!!
            val tracks = logic.search(query, call.getAuthorizationData())
            call.respond(tracks)
        }

        post("/") {
            val request = call.receive<ImportTrackRequest>()
            val track = logic.save(request.source, request.trackId, call.getAuthorizationData())
            call.respond(HttpStatusCode.Created, track)
        }

        delete("/trackId/{source}/{trackId}") {
            val source = call.parameters["source"]
            val trackId = call.parameters["trackId"]

            if (source != null && trackId != null) {
                logic.deleteByTrackId(trackId, source)
                call.respond(HttpStatusCode.NoContent)
            } else {
                call.respond(HttpStatusCode.BadRequest)
            }
        }
    }
}
