package dev.fyloz.musicplayer.core.logic

import dev.fyloz.musicplayer.core.data.TrackRepository
import dev.fyloz.musicplayer.core.factory.TrackFactoryProxy
import dev.fyloz.musicplayer.core.http.auth.AuthorizationData
import dev.fyloz.musicplayer.core.model.SearchResult
import dev.fyloz.musicplayer.core.model.Track
import org.koin.core.component.KoinComponent
import org.koin.core.component.inject
import java.util.UUID

class TrackLogic : KoinComponent {
    private val repository by inject<TrackRepository>()
    private val trackFactory by inject<TrackFactoryProxy>()

    fun getAll() = repository.findAll()
    fun getAllTrackIdsBySource() = getAll()
        .groupBy { it.source }
        .mapValues { it.value.map { track -> track.trackId } }

    fun getById(id: String) = repository.findById(id)

    suspend fun search(query: String, auth: AuthorizationData): SearchResult {
        return trackFactory.search(query, auth)
    }

    suspend fun save(source: String, trackId: String, auth: AuthorizationData): Track {
        val id = generateId()
        val track = trackFactory.create(source, id, trackId, auth)
        repository.save(track)
        return track
    }

    fun deleteByTrackId(trackId: String, source: String) {
        repository.deleteByTrackId(trackId, source)
    }

    private fun generateId() = UUID.randomUUID().toString()
}
