package dev.fyloz.musicplayer.core.model

import kotlinx.serialization.Serializable

/**
 * A generic track.
 */
abstract class Track {
    /** The id of the track in the local system. **/
    abstract val id: String

    /** The id of the track in the remote service. **/
    abstract val trackId: String

    /** The name of the track. **/
    abstract val name: String

    /** The name of the authors of the track. **/
    abstract val authors: Collection<String>

    /** The name of the source module of the track. **/
    abstract val source: String

    /** The URLs to the track images. **/
    abstract val imagesUrls: TrackImagesUrls
}

@Serializable
data class TrackImagesUrls(
    /** The URL of the track icon. **/
    val icon: String,

    /** The URL of the track thumbnail. **/
    val thumbnail: String,

    /** The URL of the track's full resolution image. **/
    val fullRes: String
)
