package dev.fyloz.musicplayer.modules.spotify

import dev.fyloz.musicplayer.core.factory.TrackFactory
import dev.fyloz.musicplayer.core.http.auth.AuthorizationData
import dev.fyloz.musicplayer.core.model.ExternalTrack
import dev.fyloz.musicplayer.core.model.Track
import dev.fyloz.musicplayer.core.model.TrackImagesUrls

class SpotifyTrackFactory : TrackFactory {
    private val apiProvider = SpotifyApiProvider()

    override suspend fun search(query: String, auth: AuthorizationData): Collection<ExternalTrack> {
        val tracks = apiProvider.search(query, "track", auth.accessToken)

        return tracks.map {
            val artists = it.artists.map { artist -> artist.name }
            val thumbnailUrl = it.album.images.first { image -> image.width == 300 }.url

            ExternalTrack(it.id, it.name, it.album.name, artists, thumbnailUrl, it.previewUrl)
        }
    }

    override suspend fun create(id: String, trackId: String, auth: AuthorizationData): Track {
        val spotifyApiTrack = apiProvider.getTrackById(trackId, auth.accessToken)
        val artists = spotifyApiTrack.artists.map { it.name }
        val imagesUrls = TrackImagesUrls(
            spotifyApiTrack.album.images.first { it.width == 64 }.url,
            spotifyApiTrack.album.images.first { it.width == 300 }.url,
            spotifyApiTrack.album.images.first { it.width == 640 }.url
        )

        return SpotifyTrack(id, trackId, spotifyApiTrack.name, artists, imagesUrls)
    }

    private val AuthorizationData.accessToken
        get() = getModuleData<SpotifyAuthorizationData>(SpotifyModule.moduleName).accessToken
}
